#!/usr/bin/env python3
import sys, argparse, random
# Generate a single testcase for average rank

parser = argparse.ArgumentParser(description="Generator for Average Rank.")
parser.add_argument('-o', metavar='O', type=str, default="random",
                    help='Options for generation')
parser.add_argument('-n', metavar='N', type=int, required=True,
                    help='number of participants')
parser.add_argument('-d', metavar='D', type=int, required=True,
                    help='number of days')
parser.add_argument('-p', metavar='P', type=int, required=True,
                    help='number of points scored')
parser.add_argument('-s', metavar='S', type=int, default=None,
                    help='seed for generation (optional)')

args = parser.parse_args()
random.seed(args.s)

options = args.o
n = args.n
d = args.d
p = args.p

p = min(p, n*d)

answer = [[] for i in range(d)]
score = [0] * n

if "close_race" in options:
    # Every second day half of competitors get a point,
    # the next day the rest will get a point, and so on...
    current_p = 0
    for i in range(0,d):
        if current_p == p:
            continue
        if i%2 == 0:
            win = random.sample(range(0,n), n//2)
            for j in win:
                if current_p == p:
                    break
                answer[i].append(j)
                score[j] += 1
                current_p += 1
        else:
            for j in range(0,n):
                if current_p == p:
                    break
                if score[j] == i//2:
                    answer[i].append(j)
                    score[j] += 1
                    current_p += 1
else:
    # Default weights are uniform, so each participant will get around p/n points.
    W = [1.0] * n

    if "harmonic" in options:
        # Weights are 1, 1/2, 1/3, 1/4, ... which should give a quite even spread of points.
        permutation = list(range(0,n))
        random.shuffle(permutation)
        for i in range(0,n):
            j = permutation[i]
            W[j] = 1.0 / (i+1)
    
    if "exponential" in options:
        # First person has weight 1, next two have weight 1/2, next four have weight 1/4 and so on...
        permutation = list(range(0,n))
        random.shuffle(permutation)
        next_power = 2
        for i in range(0,n):
            if i+1 == next_power:
                next_power *= 2
            j = permutation[i]
            W[j] = 2.0 / next_power

    if "linear" in options:
        # Weights are 1,2,3,4,...
        permutation = list(range(0,n))
        random.shuffle(permutation)
        for i in range(0,n):
            j = permutation[i]
            W[j] = i + 1.0

    sumw = sum(W)
    for i in range(0,n):
        W[i] /= sumw

    # Assign p points according to the weights.
    pairs = set()
    current_p = 0

    for i in range(0,n):
        points = int(p * W[i])
        points = min(points,d)
        days = random.sample(range(0,d), points)
        for day in days:
            current_p += 1
            answer[day].append(i)
            pairs.add(day*n + i)
            score[i] += 1

    # Assign leftover points by picking a bunch of (day, person) pairs randomly.
    # Breaks after some iterations, so for dense instances it might not reach "p" (but it should be fairly close).
    iterations = 0
    limit = 10**6
    while current_p < p and iterations < limit:
        day = random.randint(0,d-1)
        person = random.randint(0,n-1)
        if day*n + person not in pairs:
            pairs.add(day*n + person)
            answer[day].append(person)
            current_p += 1
            score[person] += 1
        iterations += 1

# Shuffle and print:
for day in range(0,d):
    random.shuffle(answer[day])

assert(current_p <= p)

print(n,d)
for i in range(0,d):
    if len(answer[i]) == 0:
        print(0)
        continue
    print(len(answer[i]), end = " ")
    for j in range(0,len(answer[i])):
        answer[i][j] += 1
    print(*answer[i])


